#include "uart.h"

int main()
{
    uint8_t follower_1_dir = 0;
    uint8_t follower_1_dir_pin = 1;
    uint8_t follower_1_step_pin = 2;
    uint32_t follower_1_ramp_up_steps = 10;
    uint32_t follower_1_running_steps = 30;
    uint32_t follower_1_ramp_down_steps = 10;
    uint32_t follower_1_pulse_width = 385;
    uint32_t follower_1_offset;
    uint32_t follower_1_start_stop_speed = 14000;
    uint32_t follower_1_ramp_inc_dec = 3;
    uint32_t follower_1_int_part = 2;
    double follower_1_dec_part = 0.631578947368421;

    uint32_t copy_1_masters_pulse_width = 385;

    ////////////////////////////////////////////////////////////
    // This variable is only for testing purposes and it can be
    // deleted, after testing is completed.
    uint32_t follower_1_steps_made;
    ////////////////////////////////////////////////////////////

    int32_t follower_1_counter;
    int32_t follower_1_step_counter;

    uart_start(31, 30, 0, 115200);
    uart_print("start\r\n");

    ////////////////////////////////////////////////////////////
    // This variable is only for testing purposes and it can be
    // deleted, after testing is completed.
    follower_1_steps_made = 0;
    ////////////////////////////////////////////////////////////

    // Set DIRA as an output.
    DIRA |= 1 << follower_1_dir_pin;

    // Set DIRA as an output.
    DIRA |= 1 << follower_1_step_pin;

    // Set the OUTA register to match the desired direction of rotation.
    if(follower_1_dir == 0)
    {
        OUTA &= (~1 << follower_1_dir_pin);
    }
    else
    {
        OUTA |= 1 << follower_1_dir_pin;
    }

    // Get the current System Counter value.
    follower_1_counter = CNT;

    // Maintain a ramp up consistent with the master's ramp up.
    while(follower_1_ramp_up_steps != 0)
    {
        // Pause for a high pulse on the master's step pin.
        waitcnt(copy_1_masters_pulse_width + CNT);

        // Maintain a step count consistent with the master's steps.
        follower_1_step_counter++;

        // Determine if the follower should make a step, if so, then make it.
        // However keep in mind that all timing is critical at this point.
        // So adjust timers appropriately
        if(follower_1_step_counter % (uint32_t)follower_1_int_part == 0)
        {
            follower_1_offset = follower_1_start_stop_speed * follower_1_dec_part;

            // At this location, place the waitcnt for the time frame before
            // this follower is supposed to make a step.
            waitcnt(follower_1_offset + CNT);

            // At this location, drive the follower's step pin high for the
            // duration of this follower's pulse width (follower_1_pulse_width).
            OUTA |= 1 << follower_1_step_pin;
            waitcnt(follower_1_pulse_width + CNT);

            // At this location, drive the follower's step pin low.
            OUTA &= (~1 << follower_1_step_pin);

            ////////////////////////////////////////////////////////////
            // This variable is only for testing purposes and it can be
            // deleted, after testing is completed.
            follower_1_steps_made++;
            ////////////////////////////////////////////////////////////

            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, computations, the followers waitcnt
            // offset, the drive high instruction, and the drive low instruction.
            // The ultimate goal is to match the same time frame as utilized
            // by the master's waitcnt.
          follower_1_start_stop_speed--;
//          follower_1_start_stop_speed--;
//          follower_1_start_stop_speed--;
//          follower_1_start_stop_speed -= 3;
//          follower_1_start_stop_speed = follower_1_start_stop_speed - 3;
//          waitcnt((follower_1_start_stop_speed -= 3) + CNT);
//          waitcnt((follower_1_start_stop_speed -= follower_1_ramp_inc_dec) + CNT);
            waitcnt(follower_1_start_stop_speed + CNT);

            // Decrement ramp_up_steps
            follower_1_ramp_up_steps--;
        }
        else
        {
            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, so that it will match the master's
            // similar  waitcnt.
            waitcnt(follower_1_start_stop_speed + CNT);
//          waitcnt((follower_1_start_stop_speed -= follower_1_ramp_inc_dec) + CNT);

            // Decrement ramp_up_steps
            follower_1_ramp_up_steps--;
        }
    }

    uart_print("rampup complete\r\n");

    // Run this driver at speed consistent with the master's speed
    while(follower_1_running_steps != 0)
    {
        // Pause for a high pulse on the master's step pin.
        waitcnt(copy_1_masters_pulse_width + CNT);

        // Maintain a step count consistent with the master's steps.
        follower_1_step_counter++;

        // Determine if the follower should make a step, if so, then make it.
        // However keep in mind that all timing is critical at this point.
        // So adjust timers appropriately
        if(follower_1_step_counter % (uint32_t)follower_1_int_part == 0)
        {
            follower_1_offset = follower_1_start_stop_speed * follower_1_dec_part;

            // At this location, place the waitcnt for the time frame before
            // this follower is supposed to make a step.
            waitcnt(follower_1_offset + CNT);

            // At this location, drive the follower's step pin high for the
            // duration of this follower's pulse width (follower_1_pulse_width).
            OUTA |= 1 << follower_1_step_pin;
            waitcnt(follower_1_pulse_width + CNT);

            // At this location, drive the follower's step pin low.
            OUTA &= (~1 << follower_1_step_pin);

            ////////////////////////////////////////////////////////////
            // This variable is only for testing purposes and it can be
            // deleted, after testing is completed.
            follower_1_steps_made++;
            ////////////////////////////////////////////////////////////

            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, computations, the followers waitcnt
            // offset, the drive high instruction, and the drive low instruction.
            // The ultimate goal is to match the same time frame as utilized
            // by the master's waitcnt.
            waitcnt(follower_1_start_stop_speed + CNT);

            // Decrement running_steps
            follower_1_running_steps--;
        }
        else
        {
            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, so that it will match the master's
            // similar  waitcnt.
            waitcnt(follower_1_start_stop_speed + CNT);

            // Decrement running_steps
            follower_1_running_steps--;
        }
    }

    uart_print("running complete\r\n");

    // Maintain a ramp down consistent with the master's ramp up.
    while(follower_1_ramp_down_steps != 0)
    {
        // Pause for a high pulse on the master's step pin.
        waitcnt(copy_1_masters_pulse_width + CNT);

        // Maintain a step count consistent with the master's steps.
        follower_1_step_counter++;

        // Determine if the follower should make a step, if so, then make it.
        // However keep in mind that all timing is critical at this point.
        // So adjust timers appropriately
        if(follower_1_step_counter % (uint32_t)follower_1_int_part == 0)
        {
            follower_1_offset = follower_1_start_stop_speed * follower_1_dec_part;

            // At this location, place the waitcnt for the time frame before
            // this follower is supposed to make a step.
            waitcnt(follower_1_offset + CNT);

            // At this location, drive the follower's step pin high for the
            // duration of this follower's pulse width (follower_1_pulse_width).
            OUTA |= 1 << follower_1_step_pin;
            waitcnt(follower_1_pulse_width + CNT);

            // At this location, drive the follower's step pin low.
            OUTA &= (~1 << follower_1_step_pin);

            ////////////////////////////////////////////////////////////
            // This variable is only for testing purposes and it can be
            // deleted, after testing is completed.
            follower_1_steps_made++;
            ////////////////////////////////////////////////////////////

            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, computations, the followers waitcnt
            // offset, the drive high instruction, and the drive low instruction.
            // The ultimate goal is to match the same time frame as utilized
            // by the master's waitcnt.
            waitcnt(follower_1_start_stop_speed + CNT);
//          waitcnt((follower_1_start_stop_speed += follower_1_ramp_inc_dec) + CNT);

            // Decrement ramp_down_steps
            follower_1_ramp_down_steps--;
        }
        else
        {
            // The following timimg code needs adjustment to compensate for the
            // if/else conditional statement, so that it will match the master's
            // similar  waitcnt.
            waitcnt(follower_1_start_stop_speed + CNT);
//          waitcnt((follower_1_start_stop_speed += follower_1_ramp_inc_dec) + CNT);

            // Decrement ramp_down_steps
            follower_1_ramp_down_steps--;
        }
    }

    uart_print("rampdown complete\r\n");
    uart_print("follower_1_steps_made ");
    uart_print_dec(follower_1_steps_made);
    uart_print("\r\n");
}
